@@ -74,11 +74,17 @@ def condense_edges(g):
     nodes.extend(u for u in num2nodes.get(1,[]) if u not in node2closest)
     nodes.extend(u for u in num2nodes.get(2,[]) if u not in node2closest)
     h = g.subgraph(nodes).copy()
-    (h.add_nodes_from(v) for k,v in num2nodes.items() if k > 2)
+    [h.add_nodes_from(v) for k,v in num2nodes.items() if k > 2]
     for u,v,w,t,p in add_edges:
         h.add_edge(u,v,weight=w,type=t,path=p)
     cs = [h.subgraph(c).copy() for c in connected_components(h)]
     node2c = {u:c for c in cs for u in c.nodes()}
+    #temp = set(node2closest.keys())
+    #temp.update(node2c.keys())
+    #for u in list(g.nodes()):
+    #    if u not in temp:
+    #        print(u)
+    #    assert(u in temp)
     return cs, node2closest, node2c
 
 def condense(file_name_in,file_name_out,lengths=False,paths=False):
