@@ -94,7 +94,7 @@ def render(g,nodes,edges,file_name_out,markers=False,lines=False,host="localhost
     LatLngListener().add_to(m)
     end()
     start("Saving result to HTML file",file_name_out)
-    m.save(file_name_out)
+    # m.save(file_name_out)
     end('')
     file_size(file_name_out)
     return g,nodes
